from llm_bridge.logic.message_preprocess import document_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type, get_file_name
from llm_bridge.type.message import Message, Role, Content, ContentType


async def preprocess_messages(messages: list[Message], api_type: str) -> None:
    for message in messages:
        await extract_text_files_to_message(message, api_type)


async def extract_text_files_to_message(message: Message, api_type: str) -> None:
    for i in range(len(message.contents) - 1, -1, -1):
        content_item = message.contents[i]
        
        if content_item.type != ContentType.File:
            continue
            
        file_url = content_item.data
        file_type, sub_type = await get_file_type(file_url)
        
        if file_type != "text" and file_type != "application":
            continue

        if api_type.startswith("Gemini") and sub_type == "pdf":
            continue
            
        filename = get_file_name(file_url)
        file_text = await document_processor.extract_text_from_file(file_url)
        
        message.contents[i] = Content(
            type=ContentType.Text,
            data=f"{filename}: \n{file_text}\n"
        )


def extract_system_messages(messages: list[Message]) -> str:
    system_message = ""
    indices_to_delete = []
    
    for i, message in enumerate(messages):
        if message.role != Role.System:
            continue
            
        # Concatenate all text content from the system message
        for content_item in message.contents:
            if content_item.type == ContentType.Text:
                system_message += f"{content_item.data}\n"
                
        indices_to_delete.append(i)

    for index in sorted(indices_to_delete, reverse=True):
        del messages[index]

    return system_message



import logging
import re
from typing import AsyncGenerator, AsyncIterator

import httpx
from fastapi import HTTPException
from google.genai import types

from llm_bridge.client.implementations.gemini.gemini_response_handler import GeminiResponseHandler
from llm_bridge.client.model_client.gemini_client import GeminiClient
from llm_bridge.type.chat_response import ChatResponse

gemini_response_handler = GeminiResponseHandler()


def process_delta(completion_delta: types.GenerateContentResponse) -> ChatResponse:
    return gemini_response_handler.process_gemini_response(completion_delta)


class StreamGeminiClient(GeminiClient):
    async def generate_stream_response(self) -> AsyncGenerator[ChatResponse, None]:
        try:
            logging.info(f"messages: {self.messages}")

            response = await self.client.aio.models.generate_content_stream(
                model=self.model,
                contents=self.messages,
                config=self.config,
            )
        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))

        try:
            async for response_delta in response:
                yield process_delta(response_delta)
        except Exception as e:
            logging.exception(e)
            yield ChatResponse(error=repr(e))



import importlib.resources
import json
from typing import TypedDict


class ModelPrice(TypedDict):
    apiType: str
    model: str
    input: float
    output: float


def load_json_file(package, filename):
    with importlib.resources.open_text(package, filename) as f:
        return json.load(f)


def get_model_prices() -> list[ModelPrice]:
    return load_json_file("llm_bridge.resources", "model_prices.json")



from typing import TypeAlias

from llm_bridge.type.message import Role
from openai.types.responses import EasyInputMessageParam


OpenAIResponsesRole: TypeAlias = Role


OpenAIResponsesMessage: TypeAlias = EasyInputMessageParam

# class EasyInputMessageParam(TypedDict, total=False):
#     content: Required[Union[str, ResponseInputMessageContentListParam]]
#     """
#     Text, image, or audio input to the model, used to generate a response. Can also
#     contain previous assistant responses.
#     """
#
#     role: Required[Literal["user", "assistant", "system", "developer"]]
#     """The role of the message input.
#
#     One of `user`, `assistant`, `system`, or `developer`.
#     """
#
#     type: Literal["message"]
#     """The type of the message input. Always `message`."""


from dataclasses import dataclass

from llm_bridge.type.message import Role


# type: "base64"
@dataclass
class ImageSource:
    type: str
    media_type: str
    data: str


# type: "text"
@dataclass
class TextContent:
    type: str
    text: str


# type: "image"
@dataclass
class ImageContent:
    type: str
    source: ImageSource


@dataclass
class ClaudeMessage:
    role: Role
    content: str | list[TextContent | ImageContent]



import logging
import re

import httpx
import openai
from fastapi import HTTPException
from openai import APIStatusError
from openai.types.responses import WebSearchToolParam, Response

from llm_bridge.client.model_client.openai_client import OpenAIClient
from llm_bridge.type.chat_response import ChatResponse, Citation
from llm_bridge.type.serializer import serialize


def process_openai_responses_non_stream_response(
        response: Response
) -> ChatResponse:

    output_list = response.output

    texts: list[str] = []
    citations: list[Citation] = []

    for output in output_list:
        if output.type == "message":
            for content in output.content:
                if content.type == "output_text":
                    texts.append(content.text)
                # Citation is currently not working well in OpenAI Responses API
                if annotations := content.annotations:
                    for annotation in annotations:
                        text = content.text[annotation.start_index:annotation.end_index]

    content = "".join(texts)
    return ChatResponse(text=content, citations=citations)


class NonStreamOpenAIResponsesClient(OpenAIClient):
    async def generate_non_stream_response(self) -> ChatResponse:
        try:
            logging.info(f"messages: {self.messages}")
            response: Response = await self.client.responses.create(
                model=self.model,
                input=serialize(self.messages),
                temperature=self.temperature,
                stream=False,
                tools=self.tools,
            )

            return process_openai_responses_non_stream_response(response)
        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except openai.BadRequestError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except APIStatusError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))


from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.model_message import claude_message
from llm_bridge.type.model_message.claude_message import ClaudeMessage
from llm_bridge.type.message import Message, Role, ContentType, Content


async def convert_message_to_claude(message: Message) -> ClaudeMessage:
    role = message.role
    
    if role == Role.System:
        role = Role.User

    claude_content = []

    for content_item in message.contents:
        if content_item.type == ContentType.Text:
            text_content = claude_message.TextContent(type="text", text=content_item.data)
            claude_content.append(text_content)
        elif content_item.type == ContentType.File:
            file_url = content_item.data
            file_type, sub_type = await get_file_type(file_url)
            if file_type == "image":
                image_contents = await media_processor.get_claude_image_content_from_url(file_url)
                claude_content.append(image_contents)
                continue
            text_content = claude_message.TextContent(
                type="text",
                text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
            )
            claude_content.append(text_content)

    return ClaudeMessage(role=role, content=claude_content)



from google import genai
from google.genai import types
from google.genai._api_client import HttpOptions

from llm_bridge.client.implementations.gemini.non_stream_gemini_client import NonStreamGeminiClient
from llm_bridge.client.implementations.gemini.stream_gemini_client import StreamGeminiClient
from llm_bridge.logic.chat_generate.chat_message_converter import convert_messages_to_gemini
from llm_bridge.logic.message_preprocess.message_preprocessor import extract_system_messages
from llm_bridge.type.message import Message


async def create_gemini_client(
        messages: list[Message],
        model: str,
        temperature: float,
        stream: bool,
        api_key: str,
):
    client = genai.Client(
        api_key=api_key,
        http_options=HttpOptions(api_version='v1alpha') # Thinking
    )

    system_instruction = None
    tools = []
    thinking_config = None
    response_modalities = ['Text']

    if "image" not in model:
        system_instruction = extract_system_messages(messages) or " "
    if "image" not in model:
        tools.append(
            types.Tool(
                google_search=types.GoogleSearch()
            )
        )
    if "image" not in model:
        thinking_config = types.ThinkingConfig(include_thoughts=True)
    if "image" in model:
        response_modalities = ['Text', 'Image']

    config = types.GenerateContentConfig(
        system_instruction=system_instruction,
        temperature=temperature,
        safety_settings=[
            types.SafetySetting(
                category=types.HarmCategory.HARM_CATEGORY_HATE_SPEECH,
                threshold=types.HarmBlockThreshold.BLOCK_NONE,
            ),
            types.SafetySetting(
                category=types.HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT,
                threshold=types.HarmBlockThreshold.BLOCK_NONE,
            ),
            types.SafetySetting(
                category=types.HarmCategory.HARM_CATEGORY_HARASSMENT,
                threshold=types.HarmBlockThreshold.BLOCK_NONE,
            ),
            types.SafetySetting(
                category=types.HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT,
                threshold=types.HarmBlockThreshold.BLOCK_NONE,
            ),
            types.SafetySetting(
                category=types.HarmCategory.HARM_CATEGORY_CIVIC_INTEGRITY,
                threshold=types.HarmBlockThreshold.BLOCK_NONE,
            ),
        ],
        tools=tools,
        thinking_config=thinking_config,
        response_modalities=response_modalities,
    )

    gemini_messages = await convert_messages_to_gemini(messages)

    if stream:
        return StreamGeminiClient(
            model=model,
            messages=gemini_messages,
            temperature=temperature,
            client=client,
            config=config,
        )
    else:
        return NonStreamGeminiClient(
            model=model,
            messages=gemini_messages,
            temperature=temperature,
            client=client,
            config=config,
        )



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='LLM-Bridge',
    version='1.5.0-alpha',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'fastapi',
        'httpx',
        'tenacity',
        'openai',
        'google-genai',
        'anthropic',
        'PyMuPDF',
        'python-docx',
        'openpyxl',
        'python-pptx',
    ],
    tests_require=[
        'pytest',
        'pytest-asyncio'
        'python-dotenv',
        'protobuf'
    ],
    author='windsnow1025',
    author_email='windsnow125@gmail.com',
    description='A Bridge for LLMs',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    python_requires='>=3.12',
)
